@define-color butter1 #fce94f;
@define-color butter2 #edd400;
@define-color butter3 #c4a000;

@define-color orange1 #fcaf3e;
@define-color orange2 #f57900;
@define-color orange3 #ce5c00;

@define-color chocolate1 #e9b96e;
@define-color chocolate2 #c17d11;
@define-color chocolate3 #8f5902;

@define-color chameleon1 #8ae234;
@define-color chameleon2 #73d216;
@define-color chameleon3 #4e9a06;

@define-color skyblue1 #729fcf;
@define-color skyblue2 #3465a4;
@define-color skyblue3 #204a87;

@define-color plum1 #ad7fa8;
@define-color plum2 #75507b;
@define-color plum3 #5c3566;

@define-color scarletred1 #ef2929;
@define-color scarletred2 #cc0000;
@define-color scarletred3 #a40000;

@define-color aluminium1 #eeeeec;
@define-color aluminium2 #d3d7cf;
@define-color aluminium3 #babdb6;
@define-color aluminium4 #888a85;
@define-color aluminium5 #555753;
@define-color aluminium6 #2e3436;

GtkToolbar#toolbar_topnav, GtkToolbar#toolbar_subnav {
	border-left: 0px;
	border-right: 0px;
}

GtkToolbar#toolbar_subnav {
	border-bottom: 0px;
}

GitgSidebar:disabled,
GitgCreateTagDialog GtkTextView:disabled {
	color: @insensitive_fg_color;
}

.panels-paned.pane-separator,
.panels-paned.pane-separator:hover,
.sidebar-paned.pane-separator,
.sidebar-paned.pane-separator:hover {
	border-width: 0 1px 0 0;
	border-style: solid;
	border-color: @borders;
	background-color: @sidebar_bg;
	color: shade (@theme_bg_color, 0.9);
}

.panels-paned.pane-separator,
.panels-paned.pane-separator:hover {
	border-width: 1px 0 0 0;
}

.panels-paned.pane-separator:backdrop,
.panels-paned.pane-separator:hover:backdrop,
.sidebar-paned.pane-separator:backdrop,
.sidebar-paned.pane-separator:hover:backdrop {
	border-color: @unfocused_borders;
	background-color: @sidebar_bg_unfocused;
}

GitgHistoryPaned GtkEntry.ref_editing_entry {
	padding: 0;
	border: 0;
}

GitgHistoryPaned GtkEntry.ref_editing_entry:selected {
	background-color: lighter(@theme_selected_bg_color);
}

.progress-bin {
	background-color: shade (@theme_bg_color, 0.9);
}

.list-row .progress-bin {
	background-color: shade (@theme_base_color, 0.9);
}

.list-row:selected .progress-bin {
	background-color: shade (@theme_selected_bg_color, 1.2);
}

.branch, .remote, .tag, .stash {
	border-radius: 6px;
	border: 1px solid @theme_bg_color;
	color: @aluminium1;
}

.branch {
	background-color: @skyblue3;
}

.remote {
	background-color: @chameleon3;
}

.tag {
	background-color: @orange3;
}

.stash {
	background-color: @butter3;
}

GitgHistoryRefRow {
	border: 0;
}

.title-label {
	font-weight: bold;
}

.subject-label {
	font-weight: bold;
}

.sha1-label {
	font-size: 0.8em;
	color: @insensitive_fg_color;
	font-family: monospace;
}

.notifications {
	background-color: @theme_bg_color;
	border-top: 1px solid @borders;
}
