.gitg-file-header {
	transition: 0.1s ease-out;
	background-color: @theme_bg_color;
}

.gitg-file-header.expanded {
	transition: 0.1s ease-out;
	background-color: shade(@theme_bg_color, 0.95);
}

.gitg-diff-view-grid-files {
	border-top: 1px solid @borders;
}

.diff-lines-separator {
	border-right: 1px solid shade(@theme_bg_color, 0.95);
}

.diff-lines-gutter-border {
	border-right: 2px solid shade(@theme_bg_color, 0.95);
}

GitgDiffViewFileRendererText {
	color: @insensitive_fg_color;
}

GitgDiffViewFileRendererText.view {
	color: @theme_text_color;
}

GitgDiffView GitgDiffViewCommitDetails {
	border-bottom: 1px outset lighter(@borders);
}

textview.commit-message text {
	background-color: @theme_bg_color;
	color: @theme_text_color;
}

gitg-diffstat {
	border: 1px inset shade(@borders, 1.2);
	border-radius: 5px;
	background-color: shade(@theme_base_color, 0.975);
	padding: 1px 5px 1px 3px;
	-gitg-diffstat-bar-height: 5px;
}

gitg-diffstat.no-frame {
	border: 0;
}

gitg-diffstat.added,
gitg-diffstat.removed,
gitg-diffstat.no-frame.added,
gitg-diffstat.no-frame.removed {
	border: 0;
}

gitg-diffstat.added,
gitg-diffstat.added-only,
gitg-diffstat.no-frame.added,
gitg-diffstat.no-frame.added-only {
	background-color: #ddffdd;
	border-radius: 3px 0px 0px 3px;
}

gitg-diffstat.added.dark,
gitg-diffstat.added-only.dark {
	background-color: rgb(78, 154, 6);
}

gitg-diffstat.added:dir(rtl),
gitg-diffstat.no-frame.added:dir(rtl) {
	border-radius: 0px 3px 3px 0px;
}

gitg-diffstat.removed,
gitg-diffstat.removed-only,
gitg-diffstat.no-frame.removed,
gitg-diffstat.no-frame.removed-only {
	background-color: #ffdddd;
	border-radius: 0px 3px 3px 0px;
}

.dark gitg-diffstat.removed,
.dark gitg-diffstat.removed-only {
	background-color: rgb(164, 0, 0);
}

gitg-diffstat.removed:dir(rtl),
gitg-diffstat.no-frame.removed:dir(rtl) {
	border-radius: 3px 0px 0px 3px;
}

gitg-diffstat.removed-only,
gitg-diffstat.added-only,
gitg-diffstat.no-frame.removed-only,
gitg-diffstat.no-frame.added-only {
	border-radius: 3px;
}

GitgDiffViewOptions GtkSpinButton,
GitgDiffViewOptions GtkButton {
	padding: 3px 6px 3px 6px;
}

GitgDiffViewOptions {
	border-top: 1px solid @borders;
}

.language-frame {
	background-color: #e1e1e1;
	border-radius: 3px;
	padding: 3px;
}

.dark .language-frame {
	background-color: #535353;
}

GitgDiffImageSideBySide.old {
	border: 10px solid #ddddff;
}

GitgDiffImageSideBySide.new {
	border: 10px solid #ddffdd;
}
